@@ -1,11 +1,11 @@
-#-----------------------------
+# -----------------------------
 # -- kongodb --
 # lib module
-#-----------------------------
+# -----------------------------
 
 import re
 import uuid
-import ulid 
+import ulid
 import json
 import arrow
 import datetime
@@ -15,16 +15,19 @@ from functools import reduce
 
 # == JSON
 
+
 def _json_dumps_hook(o):
     return timestamp_to_str(o)
 
+
 def _json_loads_hook(json_dict):
-    for k,v in json_dict.items():        
-        if isinstance(v, str) and timestamp_valid(v): 
+    for k, v in json_dict.items():
+        if isinstance(v, str) and timestamp_valid(v):
             json_dict[k] = arrow.get(v)
     return json_dict
 
-def json_dumps(data:dict) -> str:
+
+def json_dumps(data: dict) -> str:
     """
     Convert data:dict to json string
 
@@ -38,7 +41,7 @@ def json_dumps(data:dict) -> str:
     return json.dumps(data, default=_json_dumps_hook)
 
 
-def json_loads(data:str) -> dict:
+def json_loads(data: str) -> dict:
     """
     Convert data:str to json string
 
@@ -47,7 +50,7 @@ def json_loads(data:str) -> dict:
 
     Returns:
         dict
-        
+
     """
     return json.loads(data, object_hook=_json_loads_hook)
 
@@ -61,6 +64,7 @@ def get_timestamp() -> arrow.Arrow:
     """
     return arrow.utcnow()
 
+
 def timestamp_to_str(dt) -> str:
     if isinstance(dt, arrow.Arrow):
         return dt.for_json()
@@ -68,6 +72,7 @@ def timestamp_to_str(dt) -> str:
         return dt.isoformat()
     return dt
 
+
 def timestamp_valid(dt_str) -> bool:
     try:
         datetime.datetime.fromisoformat(dt_str.replace('Z', '+00:00'))
@@ -75,6 +80,7 @@ def timestamp_valid(dt_str) -> bool:
         return False
     return True
 
+
 def gen_id() -> str:
     """
     Generates ULID
@@ -83,9 +89,15 @@ def gen_id() -> str:
     Returns:
       string
     """
-    return str(ulid.new()).lower() 
+    return str(ulid.new()).lower()
 
-def dict_set(obj:dict, path:str, value:Any):
+
+def chunk_list (lst:list, n:int):
+    """Yield successive n-sized chunks from lst."""
+    for i in range(0, len(lst), n):
+        yield lst[i:i + n]
+
+def dict_set(obj: dict, path: str, value: Any):
     """
     *Mutate #obj
 
@@ -105,7 +117,8 @@ def dict_set(obj:dict, path:str, value:Any):
         here = here.setdefault(key, {})
     here[keys[-1]] = value
 
-def dict_get(obj:dict, path:str, default:Any=None) -> dict:
+
+def dict_get(obj: dict, path: str, default: Any = None) -> dict:
     """
     Get a value from a dict via dot notation
 
@@ -126,7 +139,8 @@ def dict_get(obj:dict, path:str, default:Any=None) -> dict:
             return default
     return reduce(_getattr, [obj] + path.split('.'))
 
-def dict_pop(obj:dict, path:str) -> Any:
+
+def dict_pop(obj: dict, path: str) -> Any:
     """
     * Mutates #obj
 
@@ -141,7 +155,7 @@ def dict_pop(obj:dict, path:str) -> Any:
         Any - The value that was removed
     """
 
-    here = obj 
+    here = obj
     keys = path.split(".")
 
     for key in keys[:-1]:
@@ -153,6 +167,7 @@ def dict_pop(obj:dict, path:str) -> Any:
         del here[keys[-1]]
         return val
 
+
 def dict_merge(*dicts) -> dict:
     """         
     Deeply merge an arbitrary number of dicts                                                                    
@@ -179,20 +194,32 @@ def dict_merge(*dicts) -> dict:
             updated[key] = values[-1]
     return updated
 
-def flatten_dict(ddict: dict, prefix='') -> dict:
+
+def flatten_dict(_dict: dict, _str: str = '') -> dict:
     """
     To flatten a dict. Nested node will be separated by dot or separator
+    It takes in consideration dict in list and flat them.
+    Non dict stay as is
 
     Args:
-      ddict:
-      prefix:
+        ddict:
+        prefix:
     Returns:
-      dict
+        dict
+
     """
-    return {prefix + "." + k if prefix else k: v
-            for kk, vv in ddict.items()
-            for k, v in flatten_dict(vv, kk).items()
-            } if isinstance(ddict, dict) else {prefix: ddict}
+    sep = "."
+    ret_dict = {}
+    for k, v in _dict.items():
+        if isinstance(v, dict):
+            ret_dict.update(flatten_dict(v, _str=sep.join([_str, k]).strip(sep)))
+        elif isinstance(v, list):
+            _k =  ("%s.%s" % (_str, k)).strip(sep)
+            ret_dict[_k] = [flatten_dict(item) if isinstance(item, dict) else item for item in v]
+        else:
+            ret_dict[sep.join([_str, k]).strip(sep)] = v
+    return ret_dict
+
 
 def unflatten_dict(flatten_dict: dict) -> dict:
     """
@@ -206,6 +233,8 @@ def unflatten_dict(flatten_dict: dict) -> dict:
     output = {}
     for k, v in flatten_dict.items():
         path = k.split(".")
+        if isinstance(v, list):
+            v = [unflatten_dict(i2) if isinstance(i2, dict) else i2 for i2 in v]
         _set_nested(output, path, v)
     return output
 
@@ -216,5 +245,3 @@ def _get_nested_default(d, path):
 
 def _set_nested(d, path, value):
     _get_nested_default(d, path[:-1])[path[-1]] = value
-
-
